import struct
import re


def read_record(fp, typ):
    l1 = struct.unpack("<I", fp.read(4))[0]
    body = fp.read(l1)
    l2 = struct.unpack("<I", fp.read(4))[0]
    if l1 != l2:
        raise KeyError("Record is not properly close %d v %d" % (l1, l2))

    n = l1 / struct.calcsize(typ)
    body = struct.unpack("<"+typ*n, body)
    if typ == 'c':
        return ''.join(body)
    else:
        if n == 1:
            return body[0]
        else:
            return body


def guess_version(fp, inttype='i'):
    fp.seek(0)
    magic = fp.read(17).replace('\00', '')
    if magic == '\t McMule  \t':
        version = read_record(fp, 'c').strip()
        vers, intiness = re.match("v(\\d+)([NL])", version).groups()

        return int(vers), {'N': 'i', 'L': 'q'}[intiness]

    # Test v1 vs. v2
    intsize = struct.calcsize(inttype)
    fp.seek(6893 + 3*intsize)
    nrq, nrbins = read_record(fp, inttype)
    fp.seek(6925 + 5*intsize + 22*nrq + 16*nrbins*nrq)
    try:
        time = read_record(fp, 'd')
        fp.seek(0)
        return 2, inttype
    except KeyError:
        fp.seek(0)
        return 1, inttype
    except struct.error:
        fp.seek(0)
        return 1, inttype
